package elvisDoc;

import java.io.BufferedWriter;
import java.io.*;
import java.nio.channels.FileChannel;
import java.io.FileWriter;
import java.io.FileInputStream;
import java.util.Enumeration;
import java.util.Hashtable;

public class ElvisDocHTMLDoc {

	Hashtable<String, elvisDocItem> docList;
	public String root;
	private final String fileOpen = "<HTML>\n<HEAD>\n<TITLE>Elvis API Specifications</TITLE>\n</HEAD>\n<BODY>\n\n";
	private final String fileClose = "\n</BODY>\n</HTML>";
	private final String treeName = "elvisTree.html";
	private final String treeTabSize = "  ";
	private final static String HTMLDir = File.separator+"elvisAPIDoc" +File.separator;
	
	private final String topLeftPaneFile = "topLeftPane.html";
	private final String introFile = "introduction.html";
	private final String tutorialFile = "tutorial.html";
	private final String aboutFile = "about.html";
	private final String linkCaller = "@ELink";
	
	
	public ElvisDocHTMLDoc(Hashtable<String, elvisDocItem> idocList, String rootName, String outputPath) throws IOException{
		docList = idocList;
		root = outputPath + HTMLDir;
		File td = new File(root);
		td.mkdir();
		
		
		System.out.println("Output directory: " + root);

		copyPresetFiles(outputPath);
		createIndex(rootName);
		
		for(Enumeration<elvisDocItem> e = docList.elements(); e.hasMoreElements();) {
			
			createTypicalFile(e.nextElement(), docList.get(rootName));
			
			
		}
	
		
	}
	
	public void copyPresetFiles(String root) throws IOException {
		String elvisDoc = root+File.separator+"elvisDoc";
		FileChannel in = new FileInputStream(elvisDoc + File.separator + topLeftPaneFile).getChannel();
		FileChannel out = new FileOutputStream(root + HTMLDir + File.separator + topLeftPaneFile).getChannel();
		in.transferTo( 0, in.size(), out);
		
		in = new FileInputStream(elvisDoc + File.separator + introFile).getChannel();
		out = new FileOutputStream(root + HTMLDir + File.separator + introFile).getChannel();
		in.transferTo( 0, in.size(), out);
		
		in = new FileInputStream(elvisDoc + File.separator + tutorialFile).getChannel();
		out = new FileOutputStream(root + HTMLDir + File.separator + tutorialFile).getChannel();
		in.transferTo( 0, in.size(), out);
		
		in = new FileInputStream(elvisDoc + File.separator + aboutFile).getChannel();
		out = new FileOutputStream(root + HTMLDir + File.separator + aboutFile).getChannel();
		in.transferTo( 0, in.size(), out);
		
	}
	public void createTypicalFile(elvisDocItem edi, elvisDocItem rootItem) throws IOException {
		
		BufferedWriter fr = new BufferedWriter(new FileWriter(root + edi.name + ".html"));
		
		fr.write(fileOpen);
		//fr.write("<A HREF=index.html>Contents</A>");
		fr.write("<H1>" + edi.name + "</H1>");

		fr.write("\n<PRE>\n");
		writeToRoot(edi, fr, rootItem.name);
		fr.write("\n</PRE>\n");
		
		fr.write("\n\n\n<H2><b>Description</b></H2>");
		fr.write(insertLinks(edi.getDescription(false)) + "\n\n");
		fr.write("<b><H2>Attributes</H2></b>");
		
		
		for(Enumeration<elvisDocAttribute> e = edi.attrs.elements(); e.hasMoreElements();) {
			elvisDocAttribute ca = e.nextElement();
			fr.write("<b>" + ca.name + "</b><br>\n");
			fr.write("&nbsp;&nbsp;&nbsp;&nbsp;" + insertLinks(ca.getDescription(false)) + "<br>\n\n");
			fr.write("&nbsp;&nbsp;&nbsp;&nbsp;Default Value: " + ca.defaultValue + "<br><br>\n\n");
		}
		
		
		fr.write(fileClose);
		fr.close();
		
		
	}
	
	// Removes all link callers from the inputted string and changes
	// them to references to HTML documents.
	public String insertLinks(String in) {
		
		if(in == null) return in;
		
		while(in.contains(linkCaller)) {
			
			int linkSpot = in.indexOf(linkCaller) + linkCaller.length() + 1;
			String linkName = in.substring(linkSpot, in.indexOf(" ", linkSpot));
			
			linkName = "<A HREF=\"" + linkName + ".html\">" + linkName + "</A>";
			
			in = in.substring(0, in.indexOf(linkCaller)) + linkName + in.substring(in.indexOf(" ", linkSpot), in.length());
			
		}
		return in;
		
	}
	
	
	public void writeToRoot(elvisDocItem edi, BufferedWriter fr, String rootName) throws IOException{
		
		int lastCount = 0;
		String[] treeItems = new String[10];
		
		elvisDocItem curItem = edi;
		while(!curItem.name.equals(rootName)) {
			
			treeItems[lastCount] = "-><A HREF=" + curItem.name + ".html>" + curItem.name + "</A>";
			curItem = curItem.parent;
			lastCount++;
			
		}
		treeItems[lastCount] = "<A HREF=" + rootName + ".html>" + curItem.name + "</A>";
		
		String indent = "";
		for(int x = lastCount; x >=0; x--) {
			fr.write(indent + treeItems[x] + "\n");
			indent = indent + "     ";
		}
		
	}
	
	
	public void createIndex(String rootName) throws IOException {
		
	
		BufferedWriter fr = new BufferedWriter(new FileWriter(root + "index.html"));
		
		fr.write("<HTML>\n<HEAD>\n<TITLE>Elvis API Specifications</TITLE>\n</HEAD>\n\n\n<FRAMESET cols=\"160, 80%\">\n  <FRAMESET rows=\"173, 60%\">\n    <FRAME name=\"intro\" src=\"" + topLeftPaneFile + "\">\n    <FRAME name=\"main\" src=\"" + treeName + "\">\n  </FRAMESET>\n  <FRAME name=\"data\" src=\"" + rootName + ".html\">\n</FRAMESET>\n\n</HTML>");
		fr.close();
		
		fr = new BufferedWriter(new FileWriter(root + treeName));
		fr.write(fileOpen);
		fr.write("\n<PRE>\n");
		writeTreeRec(docList.get(rootName), fr, "");
		fr.write("\n</PRE>\n");
		fr.write(fileClose);
		fr.close();
		
		
	}
	public void writeTreeRec(elvisDocItem edi, BufferedWriter fr, String indent) throws IOException{
		
		fr.write(indent + "<A HREF=" + edi.name + ".html target=\"data\">" + edi.name + "</A><BR>");
		
		Enumeration<elvisDocItem> eChildren = edi.children.elements();
		
		while(eChildren.hasMoreElements()) {
			
			elvisDocItem curChild = eChildren.nextElement();
			
			//fr.write("<A HREF=" + root + curChild.name + ".html>" + indent + curChild.name + "<\\A><BR>");)
			
			writeTreeRec(curChild, fr, indent + treeTabSize);
		}		
	}
	
}
