package errors;

import types.Widget;

/** Abstract superclass for errors caused by attributes. */
public abstract class BadAttributeException extends GUIFileException {
    /** The widget whose attribute caused the error. */
    public final Widget widget;
    /** The name of the erroneous attribute. */
    public final String attrName;

    /** @param msg The error message. (Should be computed by the subclass.)
     *  @param widget The widget whose attribute caused the error.
     *  @param attrName The name of the erroneous attribute. */
    public BadAttributeException(String msg, Widget widget, String attrName) {
        super(msg, getLineNum(widget, attrName));
        this.widget = widget;
        this.attrName = attrName;
    }
    
    private static int getLineNum(Widget widget, String attrName) {
        final Integer ans = widget.attrLineNums.get(attrName);
        return ans != null ? ans : -1;
    }
}
