package interpreter;

import util.CommandLineParser.OptionSpec;
import util.CommandLineParser.Usage;

@Usage("usage: java Assembler [opts] file language")
enum CommandLineOptions implements OptionSpec<CommandLineOptions.State> {
    @Option(name="help", abbr={'h', '?'}, desc="show this help text")
    HELP {
        public void execute(String arg, State state) {
            state.showHelp = true;
        }
    },
    @Option(name="usage", abbr='u', desc="show usage line")
    USAGE {
        public void execute(String arg, State state) {
            state.showUsage = true;
        }
    },
    @Option(name="interactive", abbr='i',
            desc="interactively override project settings")
    INTERACTIVE {
        public void execute(String arg, State state) {
            state.interactive = true;
        }
    },
    @Option(name={"suppress-modifiers", "sMod"}, abbr='s',
            desc="suppress generation of runtime widget modifiers")
    SUPPRESS_MODIFIERS {
        public void execute(String arg, State state) {
            state.suppressModifiers = true;
        }
    },
    @Option(name={"generate-modifiers", "gMod"}, abbr='g',
            desc="generate runtime widget modifiers (the default)")
    GENERATE_MODIFIERS {
        public void execute(String arg, State state) {
            state.suppressModifiers = false;
        }
    },
    @Option(name={"constructable", "no-standalone"}, abbr='c',
            desc="generate class to be constructed by another class" + 
                " (the default)")
    CONSTRUCTABLE {
        public void execute(String arg, State state) {
            state.generateMain = false;
        }
    },
    @Option(name={"executable", "standalone"}, abbr='e',
            desc="generate class to be run on its own")
    EXECUTABLE {
        public void execute(String arg, State state) {
            state.generateMain = true;
        }
    },
    @Option(name={"output", "project-name"}, abbr='o',
            arg=ArgSpec.REQUIRED, argName="NAME",
            desc="set output class name to NAME")
    PROJECT_NAME {
        public void execute(String arg, State state) {
            state.projectName = arg;
        }
    },
    @Option(name={"package", "packageId"}, abbr='p',
            arg=ArgSpec.REQUIRED, argName="PACKAGE",
            desc="declare classes to be in PACKAGE")
    PACKAGE_ID {
        public void execute(String arg, State state) {
            state.packageId = arg;
        }
    },
    ;
    static class State {
        boolean showHelp = false;
        boolean showUsage = false;
        boolean interactive = false;
        boolean suppressModifiers = false;
        boolean generateMain = false;
        String projectName = null;
        String packageId = null;
    }
    public abstract void execute(String arg, State state);
}
