package lib.java;

import java.util.*;

import errors.BadAttributeException;

import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.Derived;

import util.Builders;

/*
 * @EName BorderLayout A layout which places children on each of the four
 * cardinal directions and in the center.
 */
@Derived(Layout.class)
public class BorderLayout extends WidgetDef {
    public BorderLayout(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }
    
    public String getLanguage() {
        return "Java";
    }

    public String preamble() {
        return String.format(
                "%s = new BorderLayout();\n", id);
    }
    
    private enum Region {
        north("PAGE_START"), south("PAGE_END"), 
        west("LINE_START"), east("LINE_END"),
        center("CENTER");
        protected final String areaId;
        Region(String areaId) { this.areaId = areaId; }
    }
    
    private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            .put("gapHoriz", "3")  // @EDefaultValue gapHoriz 3
            .put("gapVert", "3") // @EDefaultValue gapVert 3
            .finish();
    
    public Map<String, List<String>> getDefaults() {
        return defaults; 
    }
    
    private static final Map<String, List<String>> childDefaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            .put("region", "center")
            .finish();
    
    public Map<String, List<String>> getChildDefaults() {
        return childDefaults;
    }
    
    public String addAttributes(GUIAttributes attrs)
            throws BadAttributeException {
        return String.format("%s.setVgap(%d);\n" + "%1$s.setVgap(%d);",
                id, attrs.getInteger("gapHoriz"), attrs.getInteger("gapVert"));
    }

    @Override
    public String addChild(String childType, String childId,
            GUIAttributes attrs) throws BadAttributeException {
        return String.format(
                "SPECIAL$%s$layoutChildren.add(%s);\n" +
                "SPECIAL$%1$s$layoutConstraints.add(BorderLayout.%s);\n",
                parent.id, childId, 
                attrs.getEnum("region", Region.class).areaId);
        
    }
    
    private static final Set<String> imports = Builders.makeCollection(
            new TreeSet<String>(), 
            "java.awt.BorderLayout"
    );
    
    public final Set<String> getImports() {
        return imports;
    }

    /* @see types.WidgetDef#getModifiers() */
    @Override
    public final List<String> getModifiers() {
        return Collections.emptyList();
    }

    @Override
    public final List<String> getVariables() {
        return Collections.singletonList("public BorderLayout " + id + ";\n");
    }
}