package lib.java;

import java.util.*;

import output.java.CodeBuilder;

import errors.BadAttributeException;
import errors.BadChildWidgetException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.*;
import util.Builders;

/**
 * @EName Abstract base for any widget containing other widgets, such as a Window
 * or Frame.
 */
@Abstract
public class Container extends WidgetDef {

    public Container(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

    public String getLanguage() {
        return "Java";
    }

    public String preamble() {
        return "";
    }

    @Override
    public String postamble() {
        CodeBuilder builder = new CodeBuilder();
        builder.beginBlock("if (AUX$%s$hasLayout)", id)
            // .append("%s.setLayout(AUX$%<s$layout);", id)
            .beginBlock("for (int $_ix = 0; " +
                    "$_ix < SPECIAL$%s$layoutChildren.size(); $_ix++)", id)
            .append("Component $_layoutChild = " +
                    "SPECIAL$%s$layoutChildren.get($_ix);\n", id)
            .append("Object $_layoutConstraint = " +
                    "SPECIAL$%s$layoutConstraints.get($_ix);\n", id)
            .beginBlock("if ($_layoutConstraint == null)")
            .append("%s.add($_layoutChild);\n", id)
            .chainBlock("else")
            .append("%s.add($_layoutChild, $_layoutConstraint);", id)
            .finishBlock()
            .finishBlock();
        
        return builder.toString();
    }
    
    public Map<String, List<String>> getDefaults() {
        return Collections.emptyMap();
    }

    public String addAttributes(GUIAttributes attrs)
            throws BadAttributeException {
        return "";
    }

    @Override
    public String addChild(String childType, String childId, GUIAttributes attrs) throws BadChildWidgetException, BadAttributeException {
        CodeBuilder builder = new CodeBuilder();
        if (isDerivedFrom(childType, Layout.class)) {
            builder.beginBlock("if (!AUX$%s$hasLayout)", id)
                .append("AUX$%s$layout = %s;", id, childId)
                .append("AUX$%s$hasLayout = true;", id)
                .finishBlock()
                .append("%s.setLayout(AUX$%<s$layout);", id)
                ;
        }
        return builder.toString();
    }
    
    public List<String> getModifiers() {
        return Collections.emptyList();
    }

    private static final Set<String> imports =
        Builders.makeCollection(new TreeSet<String>(),
                "java.awt.Component",
                "java.util.List",
                "java.awt.LayoutManager"
        );

    public Set<String> getImports() {
        return imports;
    }
    
    public List<String> getVariables() {
        return Builders.makeCollection(new ArrayList<String>(),
                String.format(
                        "private boolean AUX$%s$hasLayout = false;\n",
                            id),
                String.format(
                        "private final List<Component> " +
                        "SPECIAL$%s$layoutChildren =\n" +
                        "    new ArrayList<Component>();\n",
                            id),
                String.format(
                        "private final List<Object> " +
                        "SPECIAL$%s$layoutConstraints =\n" +
                        "    new ArrayList<Object>();\n",
                            id),
                String.format(
                        "private LayoutManager AUX$%s$layout;\n", id)
        );
    }
}