package lib.java;

import java.util.*;

import errors.BadAttributeException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import util.Builders;

/**
 * @EName A simple text label implementing Java's JLabel.
 */
public class Label extends WidgetDef {

	public Label(Widget widget, WidgetDef parent) {
		super(widget, parent);
	}
	
	@Override
	public String getLanguage() {
		return "Java";
	}

	@Override
	public String preamble() {
		return id + " = new JLabel();\n";
	}

	@Override
	public String postamble() {
		return "";
	}

	private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            .put("text", (String) null) 		//@EDefaultValue text null
            .put("verticalalignment", "-1")		//@EDefaultValue verticalalignment -1
            .put("horizontalalignment", "-1")	//@EDefaultValue horizontalalignment -1
            .finish();

    @Override
    public Map<String, List<String>> getDefaults() {
        return defaults; 
    }
    
    private static final Map<String, String> synonyms =
        Builders.buildMap(new TreeMap<String, String>())
            .put("label", "text")
            .finish();
    
    @Override
    public Map<String, String> getSynonymousAttributes() {
        return synonyms;
    }

    @Override
	public String addAttributes(GUIAttributes attrs) 
            throws BadAttributeException {
		final StringBuilder builder = new StringBuilder();
		//@EName text Specifies the text of the label.
		if (attrs.getOne("text") != null)
		    builder.append(String.format("%s.setText(\"%s\");\n",id,
		            attrs.getOne("text")));
		//@EName horizontalalignment Specifies the horizontal alignment of the label.
		if (attrs.getInteger("horizontalalignment") != -1) 
		    builder.append(String.format("%s.setHorizontalAlignment(%d);\n", 
                    id, attrs.getInteger("horizontalalignment")));
		// @EName verticalalignment Specifies the vertical alignment of the label.
        if (attrs.getInteger("verticalalignment") != -1) 
            builder.append(String.format("%s.setVerticalAlignment(%d);\n",
                    id, attrs.getInteger("verticalalignment")));
		return builder.toString();
	}

	/*
	 * This is where the defaults are defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getDefaults() has to do is return this field. 
	 */
	
	private static final ArrayList<String> modifiers = Builders.makeCollection(
			new ArrayList<String>(),
			"public String getLabelText(JLabel label) {\n" +
			"\treturn label.getText();\n" +
			"}\n" +
			
			"public void setLabelText(JLabel label, String newText) {\n" +
			"\tlabel.setText(newText);\n" +
			"}\n",
			
			"public int getLabelVerticalAlignment(JLabel label) {\n" +
			"\treturn label.getVerticalAlignment();\n" +
			"}\n" +
			
			"public void setLabelVerticalAlignment(JLabel label, int newAlignment) {\n" +
			"\tlabel.setVerticalAlignment(newAlignment);\n" +
			"}\n",
			
			"public int getLabelHorizontalAlignement(JLabel label) {\n" +
			"\treturn label.getHorizontalAlignment();\n" +
			"}\n" +
			
			"public void setLabelHorizontalAlignment(JLabel label, int newAlignment) {\n" +
			"\tlabel.setHorizontalAlignment(newAlignment);\n" +
			"}\n"
		);

	@Override
	public ArrayList<String> getModifiers() {
		return modifiers;
	}
	
	/* This is where the import code is defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getImports() has to do is return this field. */
	 private static final Set<String> imports = Builders.makeCollection(
	            new TreeSet<String>(), "import javax.swing.JLabel;\n");

	@Override
	public Set<String> getImports() {
		return imports;
	}

    @Override
    public List<String> getVariables() {
        return Collections.singletonList("public JLabel " + id + ";\n");
    }

}
