package lib.java;

import java.util.*;

import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import util.Builders;
import errors.BadAttributeException;
import errors.BadChildWidgetException;

/**
 * @EName Definition for a Java JMenuBar.  This contains one or more Menus, and each
 * menu contains one or more MenuItems.
 * 
 * NOTE: The MenuBar definition does not contain any GUIAttributes.  Instead,
 * it acts as a holder for Menus.
 * 
 * TODO:  Mnemonic is case-sensitive, and I think we want it case-insensitive.
 * 
 * @author ritza
 */
public class MenuBar extends WidgetDef {

	public MenuBar(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }
    
	@Override
	public String getLanguage() {
		return "Java";
	}

	@Override
	public String preamble() {
		return id + " = new JMenuBar();\n";
	}

	@Override
	public String postamble() {
		return "";
	}

	@Override
	/**
	 * No attributes to add.
	 */
	public String addAttributes(GUIAttributes attributes) throws BadAttributeException {
		return "";
	}

	@Override
	/**
	 * Return an empty Map object, since there are no attributes.
	 */
	public Map<String, List<String>> getDefaults() {
		return new TreeMap<String, List<String>>(); 
	}

	@Override
	public String addChild(String childType, String childName,
                GUIAttributes childAttrs)
            throws BadChildWidgetException {
		childType = childType.toLowerCase();
		if (childType.equals("menu"))
			return id.concat(".add(").concat(childName).concat(");\n");
		throw new BadChildWidgetException(widget);
	}
	
	@Override
	/**
	 * Return an empty ArrayList, since there are no attributes.
	 */
	public ArrayList<String> getModifiers() {
		return new ArrayList<String>();
	}
	
	/* This is where the import code is defined. The idea is to have a
     * private static field that gets initialized at class loading time with
     * all the code; this way it doesn't get rebuilt each time. Then all that
     * getImports() has to do is return this field. */
	 private static final Set<String> imports = Builders.makeCollection(
	            new TreeSet<String>(), "import javax.swing.JMenuBar;\n");

	@Override
	public Set<String> getImports() {
		return imports;
	}

    @Override
    public List<String> getVariables() {
        return Collections.singletonList("public JMenuBar " + id + ";\n");
    }
}