package lib.java;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import errors.BadAttributeException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.Derived;
import util.Builders;

/**
 * MenuItem for a Java JMenuItem.  MenuItems are contained in Menus.  MenuItem
 * attributes include:
 * - enabled
 * 
 * Attributes inherited from Menu include:
 * - label
 * - mnemonic
 * 
 * @author ritza
 *
 */
// @EName implements Java's JMenuItem. MenuItems are contains in Menus.

@Derived(Menu.class)
public class MenuItem extends WidgetDef {
	public MenuItem(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

	@Override
	public String getLanguage() {
		return "Java";
	}

	@Override
	public String preamble() {
		return id + " = new JMenuItem();\n";
	}

	@Override
	public String addAttributes(GUIAttributes attributes) throws BadAttributeException {
		final StringBuilder builder = new StringBuilder();
		// @EName enabled Sets whether the MenuItem is enabled.
		builder.append(String.format("%s.setEnabled(%s);\n", id,
				attributes.getBoolean("enabled")));
		return builder.toString();
	}
    private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            .put("enabled","true") // @EDefaultValue enabled true
            .finish();

	@Override
	public Map<String, List<String>> getDefaults() {
		return defaults; 
	}

    private static final ArrayList<String> modifiers =
        Builders.makeCollection(
            new ArrayList<String>(),
            "public String getMenuItemLabel(JMenuItem menuitem) {\n" +
            "\treturn menuitem.getText();\n" +
            "}\n" +
            
            "public void setMenuItemLabel(JMenuItem menuItem, String newLabel) {\n" +
            "\tmenuItem.setText(newLabel);\n" +
            "}\n");
    
	@Override
	public ArrayList<String> getModifiers() {
		return modifiers;
	}

	public static final Set<String> imports =
        Builders.makeCollection(
            new TreeSet<String>(), 
            "import javax.swing.JMenuItem;\n",
            "import java.awt.event.KeyEvent;\n");
	@Override
	public Set<String> getImports() {
		return imports;
	}

    @Override
    public List<String> getVariables() {
        return Collections.singletonList("public JMenuItem " + id + ";\n");
    }
}
