package lib.java;

import java.util.*;

import errors.BadAttributeException;
import errors.BadChildWidgetException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.Derived;
import util.Builders;

/**
 * RadioButton for Definition for a Java radio button. 
 * 
 * @author ritza
 * @see types.WidgetDef for more information about the methods.
 *
 */
// @EName Implements Java's JRadioButton
@Derived(Button.class)
public class RadioButton extends WidgetDef{
	
	public RadioButton(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

	public String getLanguage() {
		return "Java";
	}
	
	@Override
	public List<String> getVariables() {
		 return Collections.singletonList(
	                String.format("public JRadioButton %s;\n", id));
	}
	
	public String preamble() {
		return id.concat(" = new JRadioButton();\n"); 
	}

	public String postamble() {
		return "";
	}
	
	public String addAttributes(GUIAttributes attributes) throws
            BadAttributeException {
		return "";
	}
	
    private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
        // @EDefaultValue borderPainted false
        .put("borderPainted", "false")
        .finish(); 
    
    
	public Map<String,List<String>> getDefaults(){
		return defaults;
	}

	public String addChild(String childType, String childName) throws BadChildWidgetException {
		throw new BadChildWidgetException(widget);  
	}
    
	public List<String> getModifiers() {
		return Collections.emptyList();
	}

    private static final Set<String> imports = Builders.makeCollection(
            new TreeSet<String>(), 
            "import javax.swing.JRadioButton;\n");
    
	public Set<String> getImports() {
		return imports;
	}
}
