package lib.java;

import java.util.*;

import output.java.CodeBuilder;

import errors.BadAttributeException;
import errors.BadAttributeValueException;
import types.GUIAttributes;
import types.Widget;
import types.WidgetDef;
import types.WidgetDef.Contained;
import util.Builders;
import util.Functional;
import util.Pair;

//@EName Element for Layout Purposes.

@Contained
public class Spacer extends WidgetDef {

    public Spacer(Widget widget, WidgetDef parent) {
        super(widget, parent);
    }

    @Override
    public String getLanguage() {
        return "Java";
    }

    @Override
    public String preamble() {
        return "";
    }

    @Override
    public List<String> getVariables() {
        return Collections.singletonList("Component " + id + ";\n");
    }

    private static final Map<String, List<String>> defaults =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            .put("width", "0")
            .put("height", "0")
            .finish();
    
    @Override
    public Map<String, List<String>> getDefaults() {
        return defaults;
    }

    private static final Map<String, List<String>> multisynonyms =
        Builders.buildMultiMap(new TreeMap<String, List<String>>())
            .put("size", "width", "height")
            .finish();
    
    public Map<String, List<String>> getMultisynonymousAttributes() {
        return multisynonyms;
    }
    
    @Override
    public String addAttributes(final GUIAttributes attrs)
            throws BadAttributeException {
        if (attrs.getInteger("minWidth") != -1) {
            final int minSizeX, minSizeY;
            final int prefSizeX, prefSizeY;
            final int maxSizeX, maxSizeY;
            
            if (!attrs.hasValue("preferredWidth") || !attrs.hasValue("maxWidth"))
                throw new BadAttributeValueException(
                        "must also give preferredSize and maxSize", widget,
                        "minSize", Functional.quoteAndJoin(
                                attrs.getStrings("minSize"), ", "));
            
            Pair<Integer, Integer> pair;
            
            pair = attrs.getIntegerPair("minSize");
            minSizeX = pair.a; minSizeY = pair.b;
            
            pair = attrs.getIntegerPair("preferredSize");
            prefSizeX = pair.a; prefSizeY = pair.b;
            
            pair = attrs.getIntegerPair("maxSize");
            maxSizeX = pair.a; maxSizeY = pair.b;
            
            return new CodeBuilder()
                .append("%s = new Box.Filler(\n" +
                        "    new Dimension(%d, %d),\n" +
                        "    new Dimension(%d, %d),\n" +
                        "    new Dimension(%d, %d));", id,
                        minSizeX, minSizeY,
                        prefSizeX, prefSizeY,
                        maxSizeX, maxSizeY)
                .finish();
        }
        if (attrs.getInteger("preferredWidth") != -1)
            throw new BadAttributeValueException(
                    "must also give minSize and maxSize", widget,
                    "preferredSize", Functional.quoteAndJoin(
                            attrs.getStrings("preferredSize"), ", "));
        if (attrs.getInteger("maxWidth") != -1)
            throw new BadAttributeValueException(
                    "must also give minSize and preferredSize", widget,
                    "maxSize", Functional.quoteAndJoin(
                            attrs.getStrings("maxSize"), ", "));
        
        final String widthValue, heightValue;
        
        widthValue = attrs.getString("width");
        heightValue = attrs.getString("height");
        
        if (widthValue.equals("FILL"))
            if (heightValue.equals("FILL"))
                return String.format("%s = Box.createGlue();\n", id);
            else
                return String.format("%s = Box.createHorizontalGlue();\n", id);
        else
            if (heightValue.equals("FILL"))
                return String.format("%s = Box.createVerticalGlue();\n", id);
            else
                return String.format("%s = Box.createRigidArea(" +
                        "new Dimension(%d, %d));\n", id, 
                        attrs.parseInteger("width", widthValue),
                        attrs.parseInteger("height", heightValue));
    }

    @Override
    public List<String> getModifiers() {
        return Collections.emptyList();
    }

    private static final Set<String> imports =
        Builders.makeCollection(new TreeSet<String>(),
                "java.awt.Component",
                "javax.swing.Box");
    
    @Override
    public Set<String> getImports() {
        return imports;
    }

}
