package output.java;

import java.util.*;
import java.util.regex.Pattern;

import util.Functional;

class Scope {
    final State state;
    
    private final List<String> lines = new LinkedList<String>();
    
    /** The string used to indent a line of code by one increment. */
    public static final String INDENTATION = "    ";
    
    Scope(State state) {
        this.state = state;
    }
    
    @Override
    public String toString() {
        return Functional.join(lines, "\n");
    }
    
    private static final Pattern BLANK_LINES = Pattern.compile("\\n+");
    
    Scope append(String stmt) {
        if (BLANK_LINES.matcher(stmt).matches()) {
            for (int i = stmt.length(); i != 0; i--)
                appendBlankLine();
        } else
            Collections.addAll(lines, stmt.split("\\n"));
        return this;
    }
    
    Scope append(Scope scope) {
        for (String line : scope.lines)
            lines.add(scope.state.indent(line));
        return this;
    }
    
    Scope appendBlankLine() {
        lines.add("");
        return this;
    }
}
