package util;

import java.util.Arrays;
import java.util.regex.*;

/** A few utility functions for terminal output formatting. */
public final class Output {
    private static final Pattern lineStart = Pattern.compile("(?m)^");
    private static String mkSpaces(int count) {
        char chars[] = new char[count];
        Arrays.fill(chars, ' ');
        return new String(chars);
    }
    /** Indent each line of a string by a given number of spaces.
     *  @param str The string to indent.
     *  @param width The width of the indentation.
     *  @return The resulting string. */
    public static String indent(String str, int width) {
        return lineStart.matcher(str).replaceAll(mkSpaces(width));
    }
    
    /** The width of the default indentation. */
    public static final int DEFAULT_INDENT = 2;
    /** Indent each line of a string by <code>DEFAULT_INDENT</code> spaces. 
     *  @param str The string to indent.
     *  @return The resulting string. */
    public static String indent(String str) {
        return indent(str, DEFAULT_INDENT);
    }
    
    private Output() {}
}
