package util;

import java.util.*;

/** A function returning a boolean value.
 *  Used by methods of the {@link Functional} class.
 *  <p>
 *  For instance, one could create a predicate to tell if a given integer is
 *  even or odd:
 *  <p><blockquote><pre>
 *      isEven = new Predicate&lt;Integer>() {
 *          boolean f(int n) {
 *              return f % 2 == 0;
 *          }
 *      }
 *  </pre></blockquote><p>
 *  This predicate could then be passed to methods like {@link
 *  util.Functional#separate(Collection, Predicate, Collection, Collection)}
 *  (see the example there).
 *  @param <T> The argument type of the function. */
public interface Predicate<T> {
    /** This method defines the function represented by this Predicate. 
     *  @param arg The argument to the function.
     *  @return The answer. */
    boolean f(T arg);
}
